from .Proxy import Proxy
from robot.libraries.BuiltIn import BuiltIn
import sys
from robot.libraries.Screenshot import Screenshot
from robot.api import logger
import I18nListener as i18n
import ManyTranslations as ui

class RemoveFromDictionaryProxy(Proxy):
    def __init__(self, arg_format):
        arg_format[repr(['dictionary', 'keys'])] = self

    def i18n_Proxy(self, func):
        def proxy(self, dictionary, *keys):
            full_args = [str(dictionary), str(keys)]

            dict_keys_trans = i18n.I18nListener.MAP.values(list(dictionary.keys()), full_args)
            dict_have_multi_trans  = False
            for dt in dict_keys_trans:
                if len(dt) >1:
                    dict_have_multi_trans  = True

            keys_trans = i18n.I18nListener.MAP.values(keys, full_args)
            keys_have_multi_trans = False
            for lt in keys_trans:
                if len(lt) >1:
                    keys_have_multi_trans = True
                    break
            
            if keys_have_multi_trans or dict_have_multi_trans:
                RemoveFromDictionaryProxy.show_warning(self, dictionary, keys, full_args)

                i18n.I18nListener.Is_Multi_Trans = True
                
                for i, dt in enumerate(dict_keys_trans):
                    if len(dt)>1 and str(full_args)+list(dictionary.keys())[i] not in ui.UI.unique_log:
                        multi_trans_word = [list(dictionary.keys())[i]]                                
                        ui.UI.origin_xpaths_or_arguments.append(full_args)
                        ui.UI.add_trans_info(self, multi_trans_word, dt, full_args, func.__name__)
                for i, lt in enumerate(keys_trans):
                        if len(lt) > 1 and str(full_args)+keys[i] not in ui.UI.unique_log:
                            multi_trans_word = [keys[i]]     
                            ui.UI.origin_xpaths_or_arguments.append(full_args)
                            ui.UI.add_trans_info(self, multi_trans_word, lt, full_args, func.__name__)
            
            return func(self, dictionary, *keys)
        return proxy

    def show_warning(self, dictionary, keys, full_args):
        language = 'i18n in %s:\n ' %i18n.I18nListener.LOCALE
        test_name = ('Test Name: %s') %BuiltIn().get_variable_value("${TEST NAME}") + '=> Exist multiple translations of the word' + '\n'
        message_for_dict_keys = Proxy().deal_warning_message_for_list(dictionary.keys(), full_args, 'DICT_KEYS')
        message_for_keys = Proxy().deal_warning_message_for_list(keys, full_args, 'KEYS')
        if message_for_keys != '':
            message = language + test_name + message_for_dict_keys + '\n' + \
            message_for_keys + '\n' + 'You should verify translation is correct!'
            logger.warn(message)


import SeleniumLibrary
import inspect
import os
import json
import sys
import itertools
import time
import re
from glob import glob
from selenium import webdriver
from I18nMap import I18nMap
from MappingRoutesGenerator import MappingRoutesGenerator
from SeleniumLibrary.base import keyword
from robot.libraries.BuiltIn import BuiltIn
from robot.libraries.Collections import Collections
from robot.api import logger
from robot.libraries.Screenshot import Screenshot
from selenium.webdriver.remote.webelement import WebElement
from I18nTrigger import I18nTrigger #This line will initiate Ii18nTrigger

class I18nListener:
 
    ROBOT_LISTENER_API_VERSION = 2
    MAP = None
    TRANSLATION_FILE = json.loads("{}")
    LOCALE = None
    Not_SHOW_WARNING_WORDS = []
    Is_Multi_Trans=False
    SETTING_KEYS = {}
    SETTING_TRANS = {}
    SETTING_ARGS = {}

    def __init__(self, locale='en-US', language_file_path='.',not_show_warning_words='None'):
        self.is_admin_language_set=False
        self.is_ui_open=False
        self.locale = locale
        self.attrs = {}
        self.locale_dict = {'en-US':'United Kingdom - English', 
                            'ja':'日本 - 日本語', 
                            'ko':'대한민국 - 한국어', 
                            'zh-CN':'中国 - 简体中文', 
                            'zh-TW':'台灣 - 繁體中文',  
                            'de-CH':'Schweiz - Deutsch'}

        #decide language file's path. It's '.' by default. 
        if language_file_path == 'i18njson':
            language_file_path = os.path.normpath(os.path.dirname(os.path.abspath(__file__)))

        MappingRoutesGenerator().generate(language_file_path)
        for f in glob('%s/languageFiles/%s/*%s.json' % (language_file_path, locale, locale)):
            with open(f, 'r', encoding='UTF-8') as i18n_file: 
                i18n_dict = json.load(i18n_file)
            self.combine_i18n_dict(source_dict=i18n_dict, target_dict=I18nListener.TRANSLATION_FILE)
        I18nListener.MAP = I18nMap(I18nListener.TRANSLATION_FILE, locale)
        I18nListener.LOCALE = locale # for get language Ex zh-TW, zh-CN
        I18nListener.Not_SHOW_WARNING_WORDS = self.parse_not_show_warning_words(not_show_warning_words)

    '''
        append all key, value of source_dict to target_dict
        source_dict is the dict of json file like 'common-zh-TW.json'...
    '''
    def combine_i18n_dict(self, source_dict, target_dict):
        for key, value in source_dict.items():
            target_dict[key] = value
    
    def start_suite(self, name, attrs):
        if not self.is_admin_language_set: #set the admin language in the first suite start
            self.is_admin_language_set=True
            BuiltIn().set_global_variable('${language}',self.locale_dict[self.locale])
        with open("./setting.txt", 'a+') as file:
            if os.stat("./setting.txt").st_size != 0:
                file.seek(0)
                for i, line in enumerate(file.readlines()):
                    split_key_value = []
                    split_key_value=line.strip("\n").split('~')
                    read_args = split_key_value[1].split('#')
                    I18nListener.SETTING_KEYS[i] = split_key_value[2]
                    I18nListener.SETTING_TRANS[i] = split_key_value[3]
                    I18nListener.SETTING_ARGS[i] = read_args

    def parse_not_show_warning_words(self, words_string):
        if words_string == "Not_show_warning.txt":
            Not_show_warning_txt = glob('%s/Not_show_warning.txt' % (os.path.dirname(os.path.abspath(__file__))))[0]
            with open(Not_show_warning_txt, 'r', encoding='utf-8') as f:
                words_string = f.read()
        words = words_string.split('+')
        return words

    def end_suite(self, name, attrs):
        if not self.is_ui_open and I18nListener.Is_Multi_Trans:
            self.is_ui_open=True
            import ManyTranslations
            ManyTranslations.UI()


from .Proxy import Proxy
from robot.libraries.BuiltIn import BuiltIn
import sys
from robot.libraries.Screenshot import Screenshot
from robot.api import logger
import I18nListener as i18n
import ManyTranslations as ui
from SeleniumLibrary.keywords.selectelement import SelectElementKeywords

class ListSelectionShouldBeProxy(Proxy):
    def __init__(self, arg_format):
        arg_format[repr(['locator', 'expected'])] = self

    def i18n_Proxy(self, func):
        def proxy(self, locator, *expected):
            full_args = [locator, str(expected)]

            BuiltIn().import_library('SeleniumLibrary')
            locator_trans = i18n.I18nListener.MAP.locator(BuiltIn().replace_variables(locator), full_args)
            multiple_translation_words = i18n.I18nListener.MAP.get_multiple_translation_words()
            words_trans = i18n.I18nListener.MAP.values(multiple_translation_words, full_args)

            expected_trans = i18n.I18nListener.MAP.values(expected, full_args)
            expected_have_multi_trans = False
            for lt in expected_trans:
                if len(lt) >1:
                    expected_have_multi_trans = True
                    break
            
            xpath = ""
            if len(locator_trans)>1 or expected_have_multi_trans:
                ListSelectionShouldBeProxy.show_warning(self, multiple_translation_words,expected, full_args) #show warning
                for i, lt in enumerate(locator_trans):
                    xpath += '|' + lt.replace('xpath', '') if i!=0 else lt.replace('xpath', '')
                options = SelectElementKeywords._get_selected_options(self, locator)
                labels = SelectElementKeywords._get_labels(self, options)
                is_pass = False
                for lt in expected_trans:
                    for single_tran in lt:
                        if single_tran in labels:
                            is_pass = True
                            break
                
                if is_pass:
                    i18n.I18nListener.Is_Multi_Trans = True
                    
                    for i, word_trans in enumerate(words_trans):
                        if len(word_trans)>1 and str(full_args)+multiple_translation_words[i] not in ui.UI.unique_log:
                            multi_trans_word = [multiple_translation_words[i]]              
                            ui.UI.origin_xpaths_or_arguments.append(full_args)                  
                            ui.UI.add_trans_info(self, multi_trans_word, word_trans, full_args, func.__name__)
                    for i, lt in enumerate(expected_trans):
                        if len(lt) > 1 and str(full_args)+expected[i] not in ui.UI.unique_log:
                            multi_trans_word = [expected[i]]     
                            ui.UI.origin_xpaths_or_arguments.append(full_args)
                            ui.UI.add_trans_info(self, multi_trans_word, lt, full_args, func.__name__)
            else:
                xpath = locator_trans[0]
            
            all_options = SelectElementKeywords._get_options(self, locator)
            all_labels = SelectElementKeywords._get_labels(self, all_options)
            for i, lt in enumerate(expected_trans): 
                    for single_tran in lt:
                        if single_tran in all_labels:
                            expected_trans[i] = single_tran
                            break

            return func(self, BuiltIn().replace_variables(xpath), *tuple(expected_trans))
        return proxy

    def show_warning(self, multi_trans_words, expected, full_args):
        language = 'i18n in %s:\n ' %i18n.I18nListener.LOCALE
        test_name = ('Test Name: %s') %BuiltIn().get_variable_value("${TEST NAME}") + '=> Exist multiple translations of the word' + '\n'
        message_for_words = Proxy().deal_warning_message_for_list(multi_trans_words,full_args, 'MULTI_TRANS_WORDS')
        message_for_expected = Proxy().deal_warning_message_for_list(expected,full_args, 'EXPECTED')
        if message_for_words or message_for_expected:
            message = language + test_name + message_for_words + '\n' + \
                message_for_expected + '\n' +'You should verify translation is correct!'
            logger.warn(message)


from .Proxy import Proxy
from robot.libraries.BuiltIn import BuiltIn
from robot.libraries.Screenshot import Screenshot
from robot.api import logger
import I18nListener as i18n
import sys
import ManyTranslations as ui

class CountValuesInListProxy(Proxy):
    def __init__(self, arg_format):
        arg_format[repr(['list_', 'value', 'start=0', 'end=None'])] = self
        
    def i18n_Proxy(self, func):
        def proxy(self, list_, value, start=0, end=None):
            if not list_ or not value:
                return func(self, list_, value, start, end)
            
            full_args = [str(list_), value]

            list_trans = i18n.I18nListener.MAP.values(list_, full_args)
            list_have_multi_trans = False
            for lt in list_trans:
                if len(lt)>1:
                    list_have_multi_trans = True 
                    break
            value_trans = i18n.I18nListener.MAP.value(value, full_args)            

            if list_have_multi_trans or len(value_trans)>1:
                CountValuesInListProxy.show_warning(self, list_, value, full_args)

                i18n.I18nListener.Is_Multi_Trans = True  
                
                for i, lt in enumerate(list_trans):
                    if len(lt)>1 and str(full_args)+list_[i] not in ui.UI.unique_log:
                        multi_trans_word = [list_[i]]                                
                        ui.UI.origin_xpaths_or_arguments.append(full_args)
                        ui.UI.add_trans_info(self, multi_trans_word, lt, full_args, func.__name__)
                if len(value_trans) > 1 and str(full_args)+value not in ui.UI.unique_log:
                    ui.UI.origin_xpaths_or_arguments.append(full_args)
                    multi_trans_word = [value]
                    ui.UI.add_trans_info(self, multi_trans_word, value_trans,full_args, func.__name__)
            
            for i, lt in enumerate(list_trans):
                if lt[0] in value_trans:
                    list_trans[i] = value_trans
            return func(self, list_trans, value_trans, start, end)
        return proxy   
    
    def show_warning(self, list_, value, full_args):
        language = 'i18n in %s:\n ' %i18n.I18nListener.LOCALE
        test_name = ('Test Name: %s') %BuiltIn().get_variable_value("${TEST NAME}") + '=> Exist multiple translations of the word' + '\n'
        message_for_list = Proxy().deal_warning_message_for_list(list_, full_args, 'LIST')
        message_for_value = Proxy().deal_warning_message_for_one_word(value, full_args, 'VALUE')
        message = language + test_name + message_for_list + ' '*3 + '\n' + message_for_value + '\n' + 'You should verify translation is correct!' 
        if message_for_list != '' or message_for_value != '':
            logger.warn(message)                        



from .Proxy import Proxy
from robot.libraries.BuiltIn import BuiltIn
import sys
from robot.libraries.Screenshot import Screenshot
from robot.api import logger
import I18nListener as i18n
import ManyTranslations as ui

class DictionaryShouldContainItemProxy(Proxy):
    def __init__(self, arg_format):
        arg_format[repr(['dictionary', 'key', 'value', 'msg=None'])] = self
    
    def i18n_Proxy(self, func):
        def proxy(self, dictionary, key ,value, msg=None):            
            full_args = [str(dictionary), key, value]

            dict_values_trans = i18n.I18nListener.MAP.values(list(dictionary.values()), full_args)            
            dict_have_multi_trans = False 
            for dt in dict_values_trans:
                if len(dt) >1:
                    dict_have_multi_trans  = True
            value_trans = i18n.I18nListener.MAP.value(value, full_args)
            
            if len(value_trans)>1 or dict_have_multi_trans:
                DictionaryShouldContainItemProxy.show_warning(self, dictionary, value, full_args)

                if key in dictionary.keys() and dictionary[key] == value:
                    i18n.I18nListener.Is_Multi_Trans = True
                    
                    for i, dt in enumerate(dict_values_trans):
                        if len(dt)>1 and str(full_args)+list(dictionary.values())[i] not in ui.UI.unique_log:
                            multi_trans_word = [list(dictionary.values())[i]]                                
                            ui.UI.origin_xpaths_or_arguments.append(full_args)
                            ui.UI.add_trans_info(self, multi_trans_word, dt, full_args, func.__name__)
                    if len(value_trans) > 1 and str(full_args)+value not in ui.UI.unique_log:
                        multiple_translation_word = [value]     
                        ui.UI.origin_xpaths_or_arguments.append(full_args)
                        ui.UI.add_trans_info(self, multiple_translation_word, value_trans, full_args, func.__name__)
            
            dictionary = dict(zip(list(dictionary.keys()), dict_values_trans))
            return func(self, dictionary, key, value_trans,msg)
        return proxy

    def show_warning(self, dictionary, value, full_args):
        language = 'i18n in %s:\n ' %i18n.I18nListener.LOCALE
        test_name = ('Test Name: %s') %BuiltIn().get_variable_value("${TEST NAME}") + '=> Exist multiple translations of the word' + '\n'
        message_for_dict_value = Proxy().deal_warning_message_for_list(dictionary.values(),full_args, 'DICT_VALUE')
        message_for_value = Proxy().deal_warning_message_for_one_word(value, full_args,  'VALUE')
        if message_for_dict_value or message_for_value:
            message = language + test_name + message_for_dict_value + '\n' + \
                        message_for_value + '\n' +\
                        'You should verify translation is correct!'
            logger.warn(message)


from .Proxy import Proxy
from robot.libraries.BuiltIn import BuiltIn
import sys
from robot.libraries.Screenshot import Screenshot
from robot.api import logger
import I18nListener as i18n
import ManyTranslations as ui

class GetMatchCountProxy(Proxy):
    def __init__(self, arg_format):
        arg_format[repr(['list', 'pattern', 'case_insensitive=False', 'whitespace_insensitive=False'])] = self

    def i18n_Proxy(self, func):
        def proxy(self, list, pattern, case_insensitive=False, whitespace_insensitive=False):
            if not list or not pattern:
                return func(self, list, pattern, case_insensitive, whitespace_insensitive)
            
            full_args = [str(list), pattern]

            list_trans = i18n.I18nListener.MAP.values(list, full_args)
            list_have_multi_trans = False
            for lt in list_trans:
                if len(lt) >1:
                    list_have_multi_trans = True
                    break
                
            if list_have_multi_trans: 
                if 'matches' in func.__name__:
                    GetMatchCountProxy.show_warning(self, list,full_args)
                i18n.I18nListener.Is_Multi_Trans = True
                
                for i, lt in enumerate(list_trans):
                    if len(lt)>1 and str(full_args)+list[i] not in ui.UI.unique_log:
                        multi_trans_word = [list[i]]                                
                        ui.UI.origin_xpaths_or_arguments.append(full_args)
                        ui.UI.add_trans_info(self, multi_trans_word, lt, full_args, func.__name__)
            
            return func(self, list, pattern, case_insensitive, whitespace_insensitive)
        return proxy

    def show_warning(self, list, full_args):
        language = 'i18n in %s:\n ' %i18n.I18nListener.LOCALE
        test_name = ('Test Name: %s') %BuiltIn().get_variable_value("${TEST NAME}") + '=> Exist multiple translations of the word' + '\n'
        message_for_list = Proxy().deal_warning_message_for_list(list, full_args, 'LIST')
        if message_for_list :
            message = language + test_name + message_for_list + '\n' + 'You should verify translation is correct!'
            logger.warn(message)


from .Proxy import Proxy
from robot.libraries.BuiltIn import BuiltIn
import sys
from robot.libraries.Screenshot import Screenshot
from robot.api import logger
import I18nListener as i18n
import ManyTranslations as ui
from SeleniumLibrary.keywords.tableelement import TableElementKeywords

class TableCellShouldContainProxy(Proxy):
    def __init__(self, arg_format):
        arg_format[repr(['locator', 'row', 'column', 'expected', 'loglevel=\'TRACE\''])] = self

    def i18n_Proxy(self, func):
        def proxy(self, locator, row, column, expected, loglevel='TRACE'):
            full_args = [locator, row, column, expected]

            expected_trans = i18n.I18nListener.MAP.value(expected, full_args)
            locator_trans = i18n.I18nListener.MAP.locator(BuiltIn().replace_variables(locator), full_args)
            multiple_translation_words = i18n.I18nListener.MAP.get_multiple_translation_words()
            word_trans = i18n.I18nListener.MAP.values(multiple_translation_words, full_args)

            xpath = ''
            my_expected = ''
            if len(expected_trans)>1 or len(locator_trans)>1:
                TableCellShouldContainProxy.show_warning(self, multiple_translation_words, expected, full_args)

                for i, lt in enumerate(locator_trans):
                    is_pass = False
                    is_actual = BuiltIn().run_keyword_and_return_status('Get WebElement', lt)
                    if is_actual:
                        xpath += lt.replace('xpath:','')
                        content = TableElementKeywords.get_table_cell(self, lt, row, column, loglevel)
                        for et in expected_trans:
                            if et in content:
                                my_expected += et
                                is_pass = True
                                break

                    if is_pass:
                        i18n.I18nListener.Is_Multi_Trans = True
                        
                        for i, wt in enumerate(word_trans):
                            if len(wt)>1 and str(full_args)+multiple_translation_words[i] not in ui.UI.unique_log:
                                multi_trans_word = [multiple_translation_words[i]]                            
                                ui.UI.origin_xpaths_or_arguments.append(full_args)
                                ui.UI.add_trans_info(self, multi_trans_word, wt, full_args, func.__name__)
                        if len(expected_trans) > 1 and str(full_args)+expected not in ui.UI.unique_log:
                            multiple_translation_word = [expected]     
                            ui.UI.origin_xpaths_or_arguments.append(full_args)
                            ui.UI.add_trans_info(self, multiple_translation_word, expected_trans, full_args, func.__name__)
                        break
            else:
                xpath = locator_trans[0].replace('xpath:','')
                my_expected = expected_trans[0]

            return func(self, xpath, row, column, my_expected, loglevel)
        return proxy

    def show_warning(self, multiple_translation_words, expected, full_args):
        language = 'i18n in %s:\n ' %i18n.I18nListener.LOCALE
        test_name = ('Test Name: %s') %BuiltIn().get_variable_value("${TEST NAME}") + '=> Exist multiple translations of the word' + '\n'
        message_for_multi_trans_words = Proxy().deal_warning_message_for_list(multiple_translation_words, full_args, 'MULTI_TRANS_WORDS')
        message_for_expected = Proxy().deal_warning_message_for_one_word(expected, full_args, 'EXPECTED')
        if message_for_multi_trans_words or message_for_expected:
            message = language + test_name + message_for_multi_trans_words + '\n' +\
                      message_for_expected + '\n' + 'You should verify translation is correct!'
            logger.warn(message)


from robot.utils.asserts import assert_equal
from .Proxy import Proxy
from robot.libraries.BuiltIn import BuiltIn
import sys
from robot.libraries.Screenshot import Screenshot
from robot.api import logger
import itertools
import I18nListener as i18n
import ManyTranslations as ui

class ListsShouldBeEqualProxy(Proxy):
    def __init__(self, arg_format):
        arg_format[repr(['list1', 'list2', 'msg=None', 'values=True', 'names=None', 'ignore_order=False'])] = self
    
    def i18n_Proxy(self, func):
        def proxy(self, list1, list2, msg=None, values=True, names=None, ignore_order=False):
            full_args = [str(list1), str(list2)]

            list1_trans = i18n.I18nListener.MAP.values(list1, full_args)
            list2_trans = i18n.I18nListener.MAP.values(list2, full_args)

            list1_have_multi_trans  = False
            for lt in list1_trans:
                if len(lt) >1:
                    list1_have_multi_trans  = True
                    break
            list2_have_multi_trans  = False
            for lt in list2_trans:
                if len(lt) >1:
                    list2_have_multi_trans  = True
                    break            

            if list1_have_multi_trans or list2_have_multi_trans:
                ListsShouldBeEqualProxy.show_warning(self, list1, list2, full_args)

                is_pass = False
                if len(list1) == len(list2):
                    if ignore_order:
                        list1 = sorted(list1)
                        list2 = sorted(list2)
                    if assert_equal(list1, list2) == None:
                        is_pass = True

                if is_pass :
                    i18n.I18nListener.Is_Multi_Trans = True
                    
                    for i, lt in enumerate(list1_trans):
                        if len(lt)>1 and str(full_args)+list1[i] not in ui.UI.unique_log:
                            multi_trans_word = [list1[i]]                            
                            ui.UI.origin_xpaths_or_arguments.append(full_args)
                            ui.UI.add_trans_info(self, multi_trans_word, lt, full_args, func.__name__)
                    for i, lt in enumerate(list2_trans):
                        if len(lt)>1 and str(full_args)+list2[i] not in ui.UI.unique_log:
                            multi_trans_word = [list2[i]]                            
                            ui.UI.origin_xpaths_or_arguments.append(full_args)
                            ui.UI.add_trans_info(self, multi_trans_word, lt, full_args, func.__name__)
                            
            return func(self, list1_trans, list2_trans, msg, values, names, ignore_order)
        return proxy
    
    def show_warning(self, list1, list2, full_args):
        language = 'i18n in %s:\n ' %i18n.I18nListener.LOCALE
        test_name = ('Test Name: %s') %BuiltIn().get_variable_value("${TEST NAME}") + '=> Exist multiple translations of the word' + '\n'
        message_for_list1 = Proxy().deal_warning_message_for_list(list1, full_args, 'LIST1')
        message_for_list2 = Proxy().deal_warning_message_for_list(list2, full_args,  'LIST2')
        if message_for_list1 or message_for_list2:
            message = language + test_name + message_for_list1 + '\n' + message_for_list2 + '\n'\
                        'You should verify translation is correct!'
            logger.warn(message)


from .Proxy import Proxy
from robot.libraries.BuiltIn import BuiltIn
import sys
from robot.libraries.Screenshot import Screenshot
from robot.api import logger
import I18nListener as i18n
import ManyTranslations as ui

class RemoveValuesFromListProxy(Proxy):
    def __init__(self, arg_format):
        arg_format[repr(['list_', 'values'])] = self
    
    def i18n_Proxy(self, func):
        def proxy(self, list_, *values):
            full_args = [str(list_), str(values)]            

            list_trans = i18n.I18nListener.MAP.values(list_, full_args)
            list_have_multi_trans  = False
            for lt in list_trans:
                if len(lt) >1:
                    list_have_multi_trans  = True

            values_trans = i18n.I18nListener.MAP.values(values, full_args)
            values_have_multi_trans = False
            for lt in values_trans:
                if len(lt) >1:
                    values_have_multi_trans = True
                    break
            
            if list_have_multi_trans or values_have_multi_trans:
                RemoveValuesFromListProxy.show_warning(self, list_, values, full_args)

                i18n.I18nListener.Is_Multi_Trans = True
                
                for i, lt in enumerate(list_trans):
                    if len(lt)>1 and str(full_args)+list_[i] not in ui.UI.unique_log:
                        multi_trans_word = [list_[i]]                                
                        ui.UI.origin_xpaths_or_arguments.append(full_args)
                        ui.UI.add_trans_info(self, multi_trans_word, lt, full_args, func.__name__)
                for i, lt in enumerate(values_trans):
                        if len(lt) > 1 and str(full_args)+values[i] not in ui.UI.unique_log:
                            multi_trans_word = [values[i]]     
                            ui.UI.origin_xpaths_or_arguments.append(full_args)
                            ui.UI.add_trans_info(self, multi_trans_word, lt, full_args, func.__name__)
                        
            for i,lt in enumerate(list_trans):
                list_[i] = lt[0] 
                           
            for i,vt in enumerate(values_trans):
                for j in range(len(vt)):
                    if vt[j] in list_:
                        values_trans[i] = vt[j]
                        break;
            
            return func(self, list_, *tuple(values_trans))
        return proxy

    def show_warning(self, list_, values, full_args):
        language = 'i18n in %s:\n ' %i18n.I18nListener.LOCALE
        test_name = ('Test Name: %s') %BuiltIn().get_variable_value("${TEST NAME}") + '=> Exist multiple translations of the word' + '\n'
        message_for_list = Proxy().deal_warning_message_for_list(list_, full_args, 'LIST')
        message_for_values = Proxy().deal_warning_message_for_list(values, full_args, 'VALUES')
        if message_for_list or message_for_values:
            message = language + test_name + message_for_list + '\n' + \
            message_for_values + '\n' + 'You should verify translation is correct!'
            logger.warn(message)


from tkinter import *
import random
import time
import threading
from PIL import Image, ImageTk
from robot.api import logger
import tkinter.font as tkFont
import os


class UI:
    keyword_name = []
    translations_key = []
    translations_value = []
    origin_xpaths_or_arguments = []
    unique_log= []

    def __init__(self):
        self.run()

    def add_trans_info(self, multi_trans_word, translations, full_args, func_name):
        if not type(translations[0]) == list:
            translations = [translations]
        
        UI.translations_key.append(multi_trans_word[0])
        UI.translations_value.append(translations[0])
        UI.unique_log.append(str(full_args) + multi_trans_word[0])
        UI.add_keyword_name(self,func_name)

    def add_keyword_name(self, func_name):
        robot_func_name = ""
        for n, i in enumerate(func_name.split('_')):
            robot_func_name = i.capitalize() if n==0 else robot_func_name+ " " + i.capitalize()
        UI.keyword_name.append(robot_func_name)

    def get_transdic_keys_and_values(self):
        if UI.translations_key and UI.translations_value:
            for key in UI.translations_key:
                self.label_texts.append(key)
            for value in UI.translations_value:
                self.radio_texts.append(value)

    def output_setting_file(self):
        with open("./setting.txt", "a") as out_file:
            contents = ""
            for i in range(len(self.label_texts)):
                now_selected = self.radio_vars[i].get()               

                format_args=""
                for j in UI.origin_xpaths_or_arguments[i]:
                    format_args += j + "#"
                format_args = format_args[:-1]

                contents += UI.keyword_name[i] + "~" + format_args + "~" + self.label_texts[i] + "~" + self.radio_texts[i][now_selected] + "\n"
            logger.warn(contents)
            out_file.write(contents)

            for i in range(len(self.labels)):
                self.labels[i].grid_forget()
                self.labels_word[i].grid_forget()
                for j in range(len(self.radios[i])):
                    self.radios[i][j].grid_forget()
            self.btn_submit.grid_forget()
    
    def undo_trans(self):
        with open("./setting.txt", "a+") as modi_file:
            if os.stat("./setting.txt").st_size != 0:
                modi_file.seek(0)
                content_rmv = []
                for i in range(len(self.checkbtn_vars)):
                    if self.checkbtn_vars[i].get() == 1:
                        content_rmv.append(self.line_record[i]+'\n')

                new_data = ""
                for line in modi_file.readlines():
                    if line not in content_rmv:
                        new_data += line
                modi_file.seek(0)
                modi_file.truncate()
                modi_file.write(new_data)       
            self.record_ui.destroy()

    def draw_trans_options(self):
        self.labels = []   #label 1

        self.labels_word = [] #label 2
        self.label_texts = []

        self.radios = []
        self.radio_vars = []
        self.radio_texts = []
        self.get_transdic_keys_and_values()
        for i in range(len(self.label_texts)):
            self.radio_vars.append(IntVar())
            self.radios.append([])

            args = " "
            for k in range(len(UI.origin_xpaths_or_arguments[i])):
                args = args + UI.origin_xpaths_or_arguments[i][k] if k==0 else args + " , " + UI.origin_xpaths_or_arguments[i][k]

            self.labels.append(Label(self.win, text="關鍵字:%s, 參數部分是:%s  " % 
            (UI.keyword_name[i] , args), font=self.fontStyle))
            self.labels[i].grid( column=0,row=i, sticky=W+N+S, padx=10, pady=3)

            self.labels_word.append(Label(self.win, text="%s可以被翻譯成: " % 
            (self.label_texts[i]), font=self.fontStyle, fg = "red"))
            self.labels_word[i].grid( column=1,row=i, sticky=W+N+S, padx=10, pady=3)

            for j in range(len(self.radio_texts[i])):
                default_value = j
                self.radios[i].append(Radiobutton(self.win, variable=self.radio_vars[i], text=self.radio_texts[i][j], font=self.fontStyle, value=default_value))
                self.radios[i][j].grid(columnspan=1, column=2+j, row=i, sticky=W+N+S, pady=3)

    def open_record(self):
        self.record_ui = Toplevel(self.win)
        self.record_ui.title("使用者翻譯紀錄")
        self.record_ui.geometry('+250+250')

        with open("./setting.txt", 'a+') as file:
            if os.stat("./setting.txt").st_size != 0:
                file.seek(0)

                self.checkbtns = []
                self.checkbtn_vars = []
                self.checkbtn_texts = []
                self.line_record = []
                for line in file.readlines():
                    self.line_record.append(line.strip('\n')) 
                    line_split = line.strip('\n').split('~')
                    present_str = "關鍵字:%s, 參數部分:%s . 待翻譯詞:%s, 翻譯:%s" %(line_split[0],line_split[1].replace('#',', '), line_split[2], line_split[3])
                    self.checkbtn_texts.append(present_str)
                
                for i in range(len(self.checkbtn_texts)):
                    self.checkbtn_vars.append(IntVar())
                    self.checkbtns.append(Checkbutton(self.record_ui, variable=self.checkbtn_vars[i], text=self.checkbtn_texts[i], font=self.fontStyle, \
                                                        bg='light green'))
                    self.checkbtns[i].grid(column=0, row=i, sticky=W+N+S, padx=10, pady=3)
            
            text_undo = StringVar()
            btn_undo = Button(self.record_ui, textvariable=text_undo, command= self.undo_trans, font=self.fontStyle, bg="#ff8a15", fg="white", height=1, width=8)
            text_undo.set("Undo")
            btn_undo.grid(row=10, column=0, sticky=S+E, padx=10, pady=5, columnspan=3)

    def run(self):
        self.win = Tk()    
        self.win.title("一詞多譯")
        self.win.geometry('+200+300')        
        self.fontStyle = tkFont.Font(family ="Helvetica", size=14)
        self.draw_trans_options()
        
        self.instructions = Label(self.win, text="Choose the translation(s) you want!!", font=self.fontStyle)
        self.instructions.grid(row=10, sticky=S+W, padx=10, pady=5)

        self.text_record = StringVar()
        self.btn_record = Button(self.win, textvariable=self.text_record, command= self.open_record, font=self.fontStyle, bg="#8c4646", fg="white", height=2, width=15)
        self.text_record.set("TransRecord")
        self.btn_record.grid(row=10, column=1, sticky=S+E, padx=10, pady=5)

        self.text_submit = StringVar()
        self.btn_submit = Button(self.win, textvariable=self.text_submit, command= lambda:self.output_setting_file(), font=self.fontStyle, bg="#20bebe", fg="white", height=2, width=15)
        self.text_submit.set("Submit")
        self.btn_submit.grid(row=10, column=2, sticky=S+E, padx=10, pady=5, columnspan=10)

        self.win.mainloop()


  
if __name__=='__main__':
    self.run()


from .Proxy import Proxy
from robot.libraries.BuiltIn import BuiltIn
import sys
from robot.libraries.Screenshot import Screenshot
from robot.api import logger
import I18nListener as i18n
import ManyTranslations as ui
from robot.utils import unic

class ListShouldContainSubListProxy(Proxy):
    def __init__(self, arg_format):
        arg_format[repr(['list1', 'list2', 'msg=None', 'values=True'])] = self
    
    def i18n_Proxy(self, func):
        def proxy(self, list1, list2, msg=None, values=True):
            full_args = [str(list1), str(list2)]

            list1_trans = i18n.I18nListener.MAP.values(list1, full_args)
            list2_trans = i18n.I18nListener.MAP.values(list2, full_args)

            list1_have_multi_trans  = False
            for lt in list1_trans:
                if len(lt) >1:
                    list1_have_multi_trans  = True
                    break
            list2_have_multi_trans  = False
            for lt in list2_trans:
                if len(lt) >1:
                    list2_have_multi_trans  = True
                    break

            if list1_have_multi_trans or list2_have_multi_trans:
                ListShouldContainSubListProxy.show_warning(self, list1, list2, full_args)

                diffs = ', '.join(unic(item) for item in list2 if item not in list1)
                if not diffs:
                    i18n.I18nListener.Is_Multi_Trans = True
                    
                    for i, lt in enumerate(list1_trans):
                        if len(lt)>1 and str(full_args)+list1[i] not in ui.UI.unique_log:
                            multi_trans_word = [list1[i]]                            
                            ui.UI.origin_xpaths_or_arguments.append(full_args)
                            ui.UI.add_trans_info(self, multi_trans_word, lt, full_args, func.__name__)
                    for i, lt in enumerate(list2_trans):
                        if len(lt)>1 and str(full_args)+list2[i] not in ui.UI.unique_log:
                            multi_trans_word = [list2[i]]                            
                            ui.UI.origin_xpaths_or_arguments.append(full_args)
                            ui.UI.add_trans_info(self, multi_trans_word, lt, full_args, func.__name__)
            return func(self, list1_trans, list2_trans, msg, values)
        return proxy

    def show_warning(self, list1, list2, full_args):
        language = 'i18n in %s:\n ' %i18n.I18nListener.LOCALE
        test_name = ('Test Name: %s') %BuiltIn().get_variable_value("${TEST NAME}") + '=> Exist multiple translations of the word' + '\n'
        message_for_list1 = Proxy().deal_warning_message_for_list(list1, full_args, 'LIST1')
        message_for_list2 = Proxy().deal_warning_message_for_list(list2, full_args,  'LIST2')
        if message_for_list1 or message_for_list2:
            message = language + test_name + message_for_list1 + '\n' + message_for_list2 + '\n'\
                        'You should verify translation is correct!'
            logger.warn(message)


import SeleniumLibrary
import re
import os
import inspect
import json
import sys
from glob import glob
import SeleniumLibrary
from selenium import webdriver
from robot.libraries.Collections import Collections
from SeleniumLibrary.base import keyword
from robot.libraries.BuiltIn import BuiltIn

class I18nTrigger:

    def __init__(self):
        self.arg_format = {}
        self.new_proxy_instance()
        self.set_proxy_func_to_library_class_func(Collections)
        self.set_proxy_func_to_library_class_func(webdriver.Chrome)
        self.set_proxy_func_to_library_class_func(BuiltIn)
        self.set_proxy_func_to_SeleniumLibrary()

    def get_module_name(self, path):
        file = re.findall('[_A-Za-z]+.py', path)
        file_name = re.findall('[_A-Za-z]+', file[0])
        module_name = file_name[0]
        return module_name

    def get_class_name(self, text):
        m = re.search('class[ A-Za-z]+', text)
        class_define =  m.group(0)
        m = re.findall('[a-zA-Z]+', class_define)
        class_name = m[1] # Fisrst is 'Class' Second is Class name
        return class_name

    def new_proxy_instance(self):
        module_names = []
        class_names = []
        for f in glob('%s\proxyContainer\*.py' % (os.path.dirname(os.path.abspath(__file__)))):
            with open(f, 'r', encoding='UTF-8') as sub_proxy:
                text = sub_proxy.read()
                module_name =  self.get_module_name(sub_proxy.name)
                if text and module_name != '__init__'and module_name != 'Proxy':
                    module_names.append(module_name)
                    class_names.append(self.get_class_name(text))

        zipped = zip ( module_names , class_names )
        for arg in zipped:
            inport_text = 'from proxyContainer import %s' %(arg[0])
            exec(inport_text)
            instance_text = '%s.%s(self.arg_format)' %(arg[0], arg[1])
            eval(instance_text)

    #get proxy
    def get_func_proxy(self, func):
        args_declaration = self.get_argument_declaration(func)
        if repr(args_declaration) in list(self.arg_format.keys()):
            return self.arg_format[repr(args_declaration)].i18n_Proxy(func)
        return func

    def get_argument_declaration(self, func):
        args = inspect.getfullargspec(func).args[1:]
        defaults = inspect.getfullargspec(func).defaults
        varargs = inspect.getfullargspec(func).varargs
        keywords = inspect.getargspec(func).keywords
        if defaults:
            defaults = ['=' + repr(default) for default in defaults]
            defaults = [''] * (len(args) - len(defaults)) + defaults
            args = list(arg[0] + arg[1] for arg in zip(args, defaults))
        if varargs:
            args.append(varargs)
        if keywords:
            args.append(keywords)
        return args

    def set_proxy_func_to_library_class_func(self, library):
        for str_method in dir(library):
            func = getattr(library, str_method)
            if repr(type(func)) == "<class 'function'>":
                setattr(library, str_method, self.get_func_proxy(func))

    # provide a surrogate of selenium keywords and functions so that they can test i18n
    ## SeleniumLibrary
    def set_proxy_func_to_SeleniumLibrary(self):
        import SeleniumLibrary
        keywords = [keyword.replace(' ', '_').lower() for keyword in SeleniumLibrary.SeleniumLibrary().keywords]
        for str_keywords_class in dir(SeleniumLibrary):
            keywords_class = getattr(locals().get(SeleniumLibrary.__name__), str_keywords_class)
            for str_method in dir(keywords_class):
                if str_method.replace(' ', '_').lower() in keywords:
                    func = getattr(keywords_class, str_method)
                    if repr(type(func)) == "<class 'function'>":
                        setattr(keywords_class, str_method, keyword(self.get_func_proxy(func)))

I18nTrigger() #import I18nTrigger that it will run this line.


from .Proxy import Proxy
from robot.libraries.BuiltIn import BuiltIn
import sys
from robot.libraries.Screenshot import Screenshot
from robot.api import logger
import I18nListener as i18n
import ManyTranslations as ui
from robot.utils import unic

class ListShouldContainValueProxy(Proxy):
    def __init__(self, arg_format):
        arg_format[repr(['list_', 'value', 'msg=None'])] = self

    def i18n_Proxy(self, func):
        def proxy(self, list_, value, msg=None):            
            full_args = [str(list_), value]
            
            list_trans = i18n.I18nListener.MAP.values(list_, full_args)
            value_trans = i18n.I18nListener.MAP.value(value, full_args)

            list_have_multi_trans = False
            for lt in list_trans:
                if len(lt) >1:
                    list_have_multi_trans  = True
                    break 

            if list_have_multi_trans or len(value_trans)>1:
                ListShouldContainValueProxy.show_warning(self, list_, value, full_args)

                is_pass = False
                if 'not' in func.__name__ :
                    if value not in list_:
                        is_pass = True
                else:
                    if value in list_:
                        is_pass = True
                if is_pass:
                    i18n.I18nListener.Is_Multi_Trans = True

                    for i, lt in enumerate(list_trans):
                        if len(lt)>1 and str(full_args)+list_[i] not in ui.UI.unique_log:
                            multi_trans_word = [list_[i]]                            
                            ui.UI.origin_xpaths_or_arguments.append(full_args)
                            ui.UI.add_trans_info(self, multi_trans_word, lt, full_args, func.__name__)
                    if len(value_trans) > 1 and str(full_args)+value not in ui.UI.unique_log:
                        multiple_translation_word = [value]     
                        ui.UI.origin_xpaths_or_arguments.append(full_args)
                        ui.UI.add_trans_info(self, multiple_translation_word, value_trans, full_args, func.__name__)
            return func(self, list_trans, value_trans, msg)
        return proxy

    def show_warning(self, list_, value, full_args):
        language = 'i18n in %s:\n ' %i18n.I18nListener.LOCALE
        test_name = ('Test Name: %s') %BuiltIn().get_variable_value("${TEST NAME}") + '=> Exist multiple translations of the word' + '\n'
        message_for_list = Proxy().deal_warning_message_for_list(list_, full_args, 'LIST')
        message_for_value = Proxy().deal_warning_message_for_one_word(value, full_args, 'VALUE')
        if message_for_list or message_for_value:
            message = language + test_name + message_for_list + '\n' + message_for_value + '\n' + 'You should verify translation is correct!'
            logger.warn(message)


from .Proxy import Proxy
from robot.libraries.BuiltIn import BuiltIn
import sys
from robot.libraries.Screenshot import Screenshot
from robot.api import logger
import I18nListener as i18n
import ManyTranslations as ui
from robot.utils import is_string, is_list_like

class ShouldContainProxy(Proxy):
    def __init__(self, arg_format):
        arg_format[repr(['container', 'item', 'msg=None', 'values=True', 'ignore_case=False'])] = self
    
    def i18n_Proxy(self, func):
        def proxy(self, container, item, msg=None, values=True, ignore_case=False):
            full_args = [str(container), item]

            container_trans = i18n.I18nListener.MAP.values(container, full_args)
            item_trans = i18n.I18nListener.MAP.value(item, full_args)

            container_have_multi_trans = False
            if is_list_like(container):
                for lt in container_trans:
                    if len(lt) >1:
                        container_have_multi_trans  = True
                        break 
            elif is_string(container):
                if len(container_trans)>1:
                    container_have_multi_trans = True

            if container_have_multi_trans or len(item_trans)>1:
                ShouldContainProxy.show_warning(self, container, item, full_args)
                
                is_pass = False
                if 'not' in func.__name__ :
                    if is_string(container):
                        container = container.lower()
                        if item not in container and (index not in container for index in item_trans):
                            is_pass=True
                    elif is_list_like(container):
                        if item not in container and (index not in container for index in item_trans):
                            is_pass = True
                else:
                    if is_string(container):
                        container = container.lower()
                        if item in container or (index in container for index in item_trans):
                            is_pass = True
                    elif is_list_like(container):
                        if item in container or (index in container for index in item_trans):
                            is_pass = True

                if is_pass:
                    i18n.I18nListener.Is_Multi_Trans = True

                    if is_list_like(container):
                        for i, lt in enumerate(container_trans):
                            if len(lt)>1 and str(full_args)+container[i] not in ui.UI.unique_log:
                                multi_trans_word = [container[i]]                            
                                ui.UI.origin_xpaths_or_arguments.append(full_args)
                                ui.UI.add_trans_info(self, multi_trans_word, lt, full_args, func.__name__)
                    elif is_string(container):
                        if len(container_trans)>1 and str(full_args)+container not in ui.UI.unique_log:
                            multiple_translation_word = [container]     
                            ui.UI.origin_xpaths_or_arguments.append(full_args)
                            ui.UI.add_trans_info(self, multiple_translation_word, container_trans, full_args, func.__name__)
                    if len(item_trans)>1 and str(full_args)+item not in ui.UI.unique_log:
                        multiple_translation_word = [item]     
                        ui.UI.origin_xpaths_or_arguments.append(full_args)
                        ui.UI.add_trans_info(self, multiple_translation_word, item_trans, full_args, func.__name__)

            if 'not' not in func.__name__ :
                if is_list_like(item_trans):
                    for it in item_trans:
                        if [it] in container_trans:
                            item_trans = [it]
                            break
                        
            return func(self, container_trans, item_trans, msg)
        return proxy                        
    
    def show_warning(self, container, item, full_args):
        language = 'i18n in %s:\n ' %i18n.I18nListener.LOCALE
        test_name = ('Test Name: %s') %BuiltIn().get_variable_value("${TEST NAME}") + '=> Exist multiple translations of the word' + '\n'
        message_for_container = Proxy().deal_warning_message_for_list(container, full_args, 'CONTAINER')
        message_for_item = Proxy().deal_warning_message_for_one_word(item, full_args, 'Expected Contain')
        if message_for_container or message_for_item :
            message = language + test_name + message_for_container + '\n' +  message_for_item + '\n' + 'You should verify translation is correct!'
            logger.warn(message)


from .Proxy import Proxy
from robot.libraries.BuiltIn import BuiltIn
import sys
from robot.libraries.Screenshot import Screenshot
from robot.api import logger
from SeleniumLibrary.keywords import AlertKeywords
import I18nListener as i18n
import ManyTranslations as ui

class AlertShouldBePresentProxy(Proxy):
    def __init__(self, arg_format):
        arg_format[repr(['text=\'\'', 'action=\'ACCEPT\'', 'timeout=None'])] = self
    def i18n_Proxy(self, func):
        def proxy(self, text='', action='ACCEPT', timeout=None):
            full_args = [text] 

            text_trans = i18n.I18nListener.MAP.value(text, full_args)

            if len(text_trans)>1:
                AlertShouldBePresentProxy.show_warning(self, text, full_args)

                pass_tt = ''
                is_pass = False
                if 'present' in func.__name__ :
                    message = AlertKeywords.handle_alert(action, timeout)
                    for tt in text_trans:
                        if tt == message:
                            is_pass = True
                            pass_tt = tt
                            break
                if is_pass:
                    i18n.I18nListener.Is_Multi_Trans = True

                    if len(text_trans)>1 and str(full_args)+text not in ui.UI.unique_log:
                        multiple_translation_word = [text]     
                        ui.UI.origin_xpaths_or_arguments.append(full_args)
                        ui.UI.add_trans_info(self, multiple_translation_word, text_trans, full_args, func.__name__)
                    return func(self, pass_tt, action, timeout)

            return func(self, text_trans[0], action, timeout)
        return proxy 

    def show_warning(self, text):
        language = 'i18n in %s:\n' %i18n.I18nListener.LOCALE
        test_name = ('Test Name: %s') %BuiltIn().get_variable_value("${TEST NAME}") + '=> Exist multiple translations of the word' + '\n'
        message_for_text = Proxy().deal_warning_message_for_one_word(text, 'Text')
        if message_for_text != '':
            message = language + test_name + message_for_text + ' '*3 + '\n' + 'You should verify translation is correct!'
            logger.warn(message)




from robot.libraries.BuiltIn import BuiltIn
from robot.api import logger
import I18nListener as i18n
import json
import re
import os
from glob import glob

class I18nMap:

    def __init__(self, translation_file,locale='en-US'):
        self.locale = locale #language
        self.translation_file = translation_file
        self.translation_mapping_routes = self.read_translation_mapping_routes()
        self.multiple_translation_words = []
        self.no_need_trans_attirbutes = ["@id", "@class"]

    def read_translation_mapping_routes(self):
        json_path = glob('./mappingRoutes.json')[0]
        with open(json_path, 'r', encoding='utf-8') as f:
            return json.load(f)
    
    def is_exist_multiple_translation_words(self, text, full_args):
        if len(self.value(text, full_args)) > 1 and text not in self.multiple_translation_words:
            self.multiple_translation_words.append(text) 
    
    '''
        new_locate_rule -> key should be the regular expression rule
                           I will use findall to find the word that is needed to translate .
                           so value should be the your match word group position.
    '''
    def locator(self, xpath, full_args, new_locate_rule={}): 
        def combine_locate_rule(rule_at, rule_bracket, locate_rule):  
            default_rule = {
                    '(('+ rule_bracket + ')\((text\(\))?\) ?= ?(\'|\")(([0-9a-zA-Z.?&()]| ?)+)(\'|\"))': 4, 
                    '(('+ rule_bracket + ')\((text\(\))?\)\ ?, ?(\'|\")(([0-9a-zA-Z.?&()]| ?)+)(\'|\"))': 4,
                    '(('+ rule_at + ') ?= ?(\'|\")(([0-9a-zA-Z.?&()]| ?)+)(\'|\"))' : 3,
                    '(('+ rule_at + ') ?, ?(\'|\")(([0-9a-zA-Z.?&()]| ?)+)(\'|\"))' : 3
                }
            if len(new_locate_rule):
                temp = dict(default_rule.items() + new_locate_rule.items())
                locate_rule = temp
            else:
                locate_rule = default_rule
            return locate_rule

        def find_all_match_word(xpath, locate_rule):  
            all_match_words = {}
            for rule in locate_rule.keys():
                matches = re.findall(rule, xpath)
                all_match_words[rule] = matches
            return all_match_words
        
        self.multiple_translation_words = []    
        if not isinstance(xpath, str):
            return [xpath]
        translated_xpath = [xpath]

        rule_for_filter = {
            "(@[a-z-]*)":"@",
            "([a-z-]*\(\))":"()"
        }
        rule_for_insert_at = ""
        rule_for_insert_bracket = ""
        all_match_attributes = find_all_match_word(xpath, rule_for_filter)
        for rule, matches in all_match_attributes.items():
            for match in matches:
                c = 0
                if match not in self.no_need_trans_attirbutes:
                    if rule_for_filter[rule] == "@":
                        rule_for_insert_at += "|" + match if c!=0 else match
                        c+=1
                    elif rule_for_filter[rule] == "()":
                        match = match.strip("()")
                        rule_for_insert_bracket += "|" + match if c!=0 else match
                        c+=1

        locate_rule = combine_locate_rule(rule_for_insert_at, rule_for_insert_bracket, new_locate_rule) 
        
        all_match_words = find_all_match_word(xpath, locate_rule)
        for rule, matches in all_match_words.items():
            for match in matches:
                match_group = locate_rule[rule]
                quot_group = match_group - 1 
                self.is_exist_multiple_translation_words(match[match_group], full_args)
                translated_xpath = self.translate(full_args, match=match[match_group], quot=match[quot_group], 
                xpaths=translated_xpath) # group 0 as self, group 4 as match, group 3 as quot 
        if xpath != list(translated_xpath)[0] :
            self.log_translation_info(xpath, translated_xpath)
        return translated_xpath
    
    def log_translation_info(self, xpath, translated_xpath):
        def is_need_to_show_warning():
            for multiple_translation_word in self.multiple_translation_words:
                if multiple_translation_word in i18n.I18nListener.Not_SHOW_WARNING_WORDS:
                    return False
            return True   
        
        def deal_translated_xpath_info(translated_xpath):
            translated_xpath_info = ''
            for i,temp_xpath in enumerate(translated_xpath):
                temp = str(i+1) + '. ' + temp_xpath + '\n   '
                translated_xpath_info = translated_xpath_info + temp
            message = 'Detail Information\ni18n in %s :\nOriginal Locator:\n   1. %s\nTranslated Locator:\n   %s' % (self.locale, xpath, translated_xpath_info)
            return message
        
        warning_or_not = is_need_to_show_warning()
        message = deal_translated_xpath_info(translated_xpath)
        if warning_or_not == False:
            words = ', '.join(self.multiple_translation_words)
            message = message + '\nYou had resolved the multiple translations of the word: \'%s\'' %(words)
        logger.info(message)

    def get_multiple_translation_words(self):
        return self.multiple_translation_words

    # Our target is "XXX" if without quot that it will translate the wrong target.
    def translate(self,full_args, match, quot, xpaths):
        origin = quot + match + quot
        translate_list = []
        for translation in self.value(match, full_args):
            value = quot + translation + quot
            for xpath in xpaths:
                translate_list.append(xpath.replace(origin, value))
        return list(set(translate_list))

    #For list should be equal, set should be equal...
    def values(self, values, full_args):
        return [self.value(v, full_args) for v in values]

    def value(self, value, full_args):
        try:
            result = self.get_possible_translation(value, full_args)
        except (KeyError):
            return [value]
        return list(set(result))

    def get_possible_translation(self, value, full_args):
        result = []
        for i in range(len(i18n.I18nListener.SETTING_KEYS)):
            if i18n.I18nListener.SETTING_KEYS[i] == value and i18n.I18nListener.SETTING_ARGS[i] == full_args:
                result.append(i18n.I18nListener.SETTING_TRANS[i])
                break
        if result:    
            return result
        else:
            try:
                for mapping_route in self.translation_mapping_routes[value]:
                    result.append(eval("self.translation_file%s" % mapping_route))
            except (KeyError):
                raise KeyError
            return result


from robot.libraries.BuiltIn import BuiltIn
import sys
from robot.libraries.Screenshot import Screenshot
import I18nListener as i18n
from robot.api import logger

class Proxy():
    def __init__(self):
        '''
            You should set your argument format as key, value is yourself.
            For example:
                arg_format[repr(['by=\'id\'', 'value=None'])] = self

            'arg_format' will pass into your constructor.
            "['by=\'id\'', 'value=None']" is your argument format.
        '''
        pass
    
    def i18n_Proxy(self, func):
        '''           
            Package keyword Proxy mapping logic by yourself.
        '''
        raise('You should implement it!')
    
    def show_warning(self):
        '''           
            Define keyword Proxy warning message by yourself.
        '''
        raise('You should implement it!')
    
    def deal_warning_message_for_one_word(self, word,full_args, message_title):
        message = ' '*3
        counter = 1
        should_return = False
        possible_translations = i18n.I18nListener.MAP.value(word, full_args)
        if len(possible_translations) > 1: 
            should_return = True
            message_value = str(counter) + '. ' + 'Multiple translations of the word: \'%s\'' %word + '\n' + ' '*6
            message_for_list = ('\'%s\'' + ' can translate to: ' + '[ ' + ', '.join(possible_translations) +' ]') %word + '\n'
            message = message + message_value + message_for_list
            counter = counter + 1 
        if should_return:
            message =  ('   %s Word:' + ' ' + '\'' + word+ '\'' + '\n' + ' '*3 + message) % message_title
        else:
            message = ''
        return message
    
    def deal_warning_message_for_list(self, deal_list, full_args, message_title):
        message = ' '*3
        counter = 1
        should_return = False
        for temp in deal_list:
            possible_translations = i18n.I18nListener.MAP.value(temp, full_args)
            if len(possible_translations) > 1: 
                should_return = True
                message_value = str(counter) + '. ' + 'Multiple translation of the word: \'%s\'' %''.join(temp) + '\n' + ' '*6
                message_for_list = ('\'%s\'' + ' can translate to: ' + '[ ' + ', '.join(possible_translations)+ ' ]') %temp
                message = message + message_value + message_for_list
                counter = counter + 1 
        list1_to_sring =  '[' + ', '.join(deal_list) + ']'
        if should_return:
            message = ('   In %s' + ' '*2 + list1_to_sring + '\n' + ' '*3 + message) % message_title
        else:
            message = ''
        return message


from .Proxy import Proxy
from robot.libraries.BuiltIn import BuiltIn
from selenium.webdriver.remote.webelement import WebElement
from robot.libraries.Screenshot import Screenshot
from robot.api import logger
import I18nListener as i18n
import sys
import ManyTranslations as ui

class FindElementsProxy(Proxy):
    def __init__(self, arg_format):
        arg_format[repr(['by=\'id\'', 'value=None'])] = self

    def i18n_Proxy(self, func):
        def proxy(self, by='id', value=None):
            if isinstance(value, WebElement):
                return func(self, by, value)
            
            full_args = [value]

            BuiltIn().import_library('SeleniumLibrary')
            locator = i18n.I18nListener.MAP.locator(BuiltIn().replace_variables(value), full_args) 
            multiple_translation_words = i18n.I18nListener.MAP.get_multiple_translation_words() 

            is_actual = False
            xpath = ''
            if len(locator) > 1:
                FindElementsProxy.show_warning(self, value, multiple_translation_words, full_args)

                for i, translation_locator in enumerate(locator):
                    xpath += '|' + translation_locator.replace('xpath:', '') if i != 0 else translation_locator.replace('xpath:', '')
                    is_actual = BuiltIn().run_keyword_and_return_status('Get WebElement', translation_locator) 
                    if is_actual:
                        break
                    
                if is_actual:
                    i18n.I18nListener.Is_Multi_Trans = True
                    
                    if str(full_args)+multiple_translation_words[0] not in ui.UI.unique_log:
                        word_translation = i18n.I18nListener.MAP.values(multiple_translation_words, full_args)
                        ui.UI.origin_xpaths_or_arguments.append(full_args)
                        ui.UI.add_trans_info(self, multiple_translation_words, word_translation, full_args, func.__name__)
                        
                    actual_locator_message = "System use the locator:'%s' to run!\n" %translation_locator
                    logger.info(actual_locator_message)
                        
            else:
                xpath = locator[0]
            return func(self, by, BuiltIn().replace_variables(xpath))
        return proxy

    def show_warning(self, locator, multiple_translation_words, full_args):
        language = 'i18n in %s:\n ' %i18n.I18nListener.LOCALE
        test_name = BuiltIn().get_variable_value("${TEST NAME}")
        message_for_words = Proxy().deal_warning_message_for_list(multiple_translation_words, full_args,  'MULTI_TRANS_WORDS')
        message = language + 'Test Name: ' + test_name + '\n' + 'locator: ' + locator +'\n'+ \
              message_for_words + '\n' + 'You should verify translation is correct!'
        if message_for_words:
            logger.warn(message)


from .Proxy import Proxy
from robot.libraries.BuiltIn import BuiltIn
import sys
from robot.libraries.Screenshot import Screenshot
from robot.api import logger
import I18nListener as i18n
import ManyTranslations as ui
from robot.libraries.Collections import _Dictionary

class DictionariesShouldBeEqualProxy(Proxy):
    def __init__(self, arg_format):
        arg_format[repr(['dict1', 'dict2', 'msg=None', 'values=True'])] = self
        
    def i18n_Proxy(self, func):
        def proxy(self, dict1, dict2, msg=None, values=True):
            full_args = [str(dict1), str(dict2)]

            dict1_keys_trans = i18n.I18nListener.MAP.values(list(dict1.keys()), full_args)
            dict1_values_trans = i18n.I18nListener.MAP.values(list(dict1.values()), full_args)
            dict2_keys_trans = i18n.I18nListener.MAP.values(list(dict2.keys()), full_args)
            dict2_values_trans = i18n.I18nListener.MAP.values(list(dict2.values()), full_args)
            whole_trans = []
            whole_trans.append(dict1_keys_trans)
            whole_trans.append(dict1_values_trans)
            whole_trans.append(dict2_keys_trans)
            whole_trans.append(dict2_values_trans)
            dict_have_multi_trans = False
            for i in range(4):
                for dt in whole_trans[i]: 
                    if len(dt)>1:
                        dict_have_multi_trans = True 
                        break

            new_dict1 = {}
            new_dict2 = {}
            new_dict2=dict(zip(list(dict2.keys()), dict2_values_trans))

            if dict_have_multi_trans:
                DictionariesShouldBeEqualProxy.show_warning(self, dict1, dict2, full_args)
                if 'contain_sub' in func.__name__:
                    keys = self.get_dictionary_keys(dict2)
                    contain_key = True  
                    for k in keys:
                        if k not in dict1:
                            contain_key = False
                            break
                    if contain_key and not list(_Dictionary._yield_dict_diffs(self, keys, dict1, dict2)):
                        diffs = False
                    else:
                        diffs = True
                elif 'equal' in func.__name__:
                    try:
                        keys = _Dictionary._keys_should_be_equal(self, dict1, dict2, msg, values)
                        diffs = list(_Dictionary._yield_dict_diffs(self, keys, dict1, dict2))
                        for k in keys:
                            if dict1[k] in dict2_values_trans[0]:
                                diffs = False
                    except:
                        for dict1_key in dict1.keys():
                            for dict2_key in new_dict2.keys():
                                if [dict1_key] in dict2_keys_trans and dict1[dict1_key] in new_dict2[dict2_key][0]:
                                    new_dict1[dict2_key] = new_dict2[dict2_key]
                                    diffs = False 
                                else:
                                    diffs = True
                                    break
                if not diffs:
                    i18n.I18nListener.Is_Multi_Trans = True

                    for i, dt in enumerate(dict1_keys_trans):
                        if len(dt)>1 and str(full_args)+list(dict1.keys())[i] not in ui.UI.unique_log:
                            ui.UI.origin_xpaths_or_arguments.append(full_args)
                            multi_trans_word = [list(dict1.keys())[i]]                                
                            ui.UI.add_trans_info(self, multi_trans_word, dt, full_args, func.__name__)
                    for i, dt in enumerate(dict1_values_trans):
                        if len(dt)>1 and str(full_args)+list(dict1.values())[i] not in ui.UI.unique_log:
                            ui.UI.origin_xpaths_or_arguments.append(full_args)
                            multi_trans_word = [list(dict1.values())[i]]                                
                            ui.UI.add_trans_info(self, multi_trans_word, dt, full_args, func.__name__)                    
                    for i, dt in enumerate(dict2_keys_trans):
                        if len(dt)>1 and str(full_args)+list(dict2.keys())[i] not in ui.UI.unique_log:
                            ui.UI.origin_xpaths_or_arguments.append(full_args)
                            multi_trans_word = [list(dict2.keys())[i]]                                
                            ui.UI.add_trans_info(self, multi_trans_word, dt, full_args, func.__name__)                    
                    for i, dt in enumerate(dict2_values_trans):
                        if len(dt)>1 and str(full_args)+list(dict2.values())[i] not in ui.UI.unique_log:
                            ui.UI.origin_xpaths_or_arguments.append(full_args)
                            multi_trans_word = [list(dict2.values())[i]]                                
                            ui.UI.add_trans_info(self, multi_trans_word, dt, full_args, func.__name__)  
            dict1 = dict(zip(list(dict1.keys()), dict1_values_trans)) 
            dict2 = dict(zip(list(dict2.keys()), dict2_values_trans))
            
            for dict1_key in dict1.keys():
                for dict2_key in dict2.keys():
                    if [dict1_key] in dict2_keys_trans and dict1[dict1_key]== dict2[dict2_key]:
                        dict1.pop(dict1_key, None)
                        dict1[dict2_key] = dict2[dict2_key]
                        return func(self, dict1, dict2)
                    elif dict1_key== dict2_key and dict1[dict1_key][0] in dict2[dict2_key]:
                        dict1[dict1_key] = dict2[dict2_key] 
            if new_dict1:
                return func(self, new_dict1, new_dict2, msg, values)     
                
            else:           
                return func(self, dict1, dict2, msg, values)                              
        return proxy

    def show_warning(self, dict1, dict2, full_args):
        language = 'i18n in %s:\n ' %i18n.I18nListener.LOCALE
        test_name = ('Test Name: %s') %BuiltIn().get_variable_value("${TEST NAME}") + '=> Exist multiple translations of the word' + '\n'
        message_for_dict1_key = Proxy().deal_warning_message_for_list(dict1.keys(), full_args, 'Dict1KEY')
        message_for_dict1_value = Proxy().deal_warning_message_for_list(dict1.values(), full_args, 'Dict1VALUE')
        message_for_dict2_key = Proxy().deal_warning_message_for_list(dict2.keys(), full_args, 'Dict2KEY')
        message_for_dict2_value = Proxy().deal_warning_message_for_list(dict2.values(), full_args, 'Dict2VALUE')
        message = language + test_name + message_for_dict1_key + '\n' + message_for_dict1_value + '\n' \
        + message_for_dict2_key + '\n' + message_for_dict2_value + '\n' + 'You should verify translation is correct!'
        if message_for_dict1_key or message_for_dict1_value or message_for_dict2_key or message_for_dict2_value:
            logger.warn(message)


from .Proxy import Proxy
from robot.libraries.BuiltIn import BuiltIn
import sys
from robot.libraries.Screenshot import Screenshot
from robot.api import logger
import I18nListener as i18n
import ManyTranslations as ui
from SeleniumLibrary.keywords.selectelement import SelectElementKeywords

class SelectFromListByValueProxy(Proxy):
    def __init__(self, arg_format):
        arg_format[repr(['locator', 'values'])] = self
    
    def i18n_Proxy(self, func):
        def proxy(self, locator, *values):
            if not values:
                return func(self, locator, values)
            full_args = [locator, str(values)]

            BuiltIn().import_library('SeleniumLibrary')
            locator_trans = i18n.I18nListener.MAP.locator(BuiltIn().replace_variables(locator), full_args)
            multiple_translation_words = i18n.I18nListener.MAP.get_multiple_translation_words()
            words_trans = i18n.I18nListener.MAP.values(multiple_translation_words, full_args)

            values_trans = i18n.I18nListener.MAP.values(values, full_args)
            values_have_multi_trans = False
            for lt in values_trans:
                if len(lt) >1:
                    values_have_multi_trans = True
                    break

            xpath = ""
            if len(locator_trans)>1 or values_have_multi_trans:
                SelectFromListByValueProxy.show_warning(self, multiple_translation_words, values, full_args) #show warning
                
                for i, lt in enumerate(locator_trans):
                    xpath += '|' + lt.replace('xpath', '') if i!=0 else lt.replace('xpath', '')

                all_options = SelectElementKeywords._get_options(self, locator)
                all_values = SelectElementKeywords._get_values(all_options)
                is_pass = False
                for lt in values_trans:
                    for single_tran in lt:
                        if single_tran in all_values:
                            is_pass = True
                            break

                if is_pass:
                    i18n.I18nListener.Is_Multi_Trans = True
                    for i, word_trans in enumerate(words_trans):
                        if len(word_trans)>1 and str(full_args)+multiple_translation_words[i] not in ui.UI.unique_log:
                            multi_trans_word = [multiple_translation_words[i]]                                
                            ui.UI.origin_xpaths_or_arguments.append(full_args)
                            ui.UI.add_trans_info(self, multi_trans_word, word_trans, full_args, func.__name__)
                    for i, lt in enumerate(values_trans):
                        if len(lt) > 1 and str(full_args)+values[i] not in ui.UI.unique_log:
                            multi_trans_word = [values[i]]     
                            ui.UI.origin_xpaths_or_arguments.append(full_args)
                            ui.UI.add_trans_info(self, multi_trans_word, lt, full_args, func.__name__)
            else:
                xpath = locator_trans[0]
            
            all_options = SelectElementKeywords._get_options(self, locator)
            all_values = SelectElementKeywords._get_values(self, all_options)
            for i, lt in enumerate(values_trans): 
                    for single_tran in lt:
                        if single_tran in all_values:
                            values_trans[i] = single_tran
                            break
        
            return func(self, BuiltIn().replace_variables(xpath), *tuple(values_trans))
        return proxy

    def show_warning(self, multi_trans_words, values, full_args):
        language = 'i18n in %s:\n ' %i18n.I18nListener.LOCALE
        test_name = ('Test Name: %s') %BuiltIn().get_variable_value("${TEST NAME}") + '=> Exist multiple translations of the word' + '\n'
        message_for_words = Proxy().deal_warning_message_for_list(multi_trans_words,full_args, 'MULTI_TRANS_WORDS')
        message_for_values = Proxy().deal_warning_message_for_list(values, full_args, 'VALUES')
        if message_for_words or message_for_values:
            message = language + test_name + message_for_words + '\n' + \
                message_for_values + '\n' +'You should verify translation is correct!'
            logger.warn(message)